"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ast_util_1 = require("typescript-ast-util");
const ts_simple_ast_1 = require("ts-simple-ast");
/**

# description

function-like is returning a value but no return type was declared in signature. suggest adding a return type inferring from return value

# example

```const other = (a: string): number => {}```

# Attacks

"code": "2355",	"message": "A function whose declared type is neither 'void' nor 'any' must return a value.",

*/
exports.addReturnStatement = {
    name: 'addReturnStatement',
    config: {},
    predicate: (arg) => {
        if (!arg.diagnostics.find(d => d.code === 2355)) {
            return false;
        }
        const kind = typescript_ast_util_1.getKindName(arg.containingTarget);
        const kindToIncludeAnyOf = ['Identifier', 'Keyword', 'Type'];
        if (kindToIncludeAnyOf.find(k => kind.includes(k))) {
            return true;
        }
        else {
            arg.log('addReturnStatement predicate false because child.kind dont include any of ' + kindToIncludeAnyOf.join(', '));
            return false;
        }
    },
    description: (arg) => `Add Return Statement`,
    apply: (arg) => {
        const firstStatementedNode = arg.simpleNode.getAncestors().find(ts_simple_ast_1.TypeGuards.isStatementedNode);
        if (firstStatementedNode) {
            addReturnStatementImpl(arg.simpleNode.getSourceFile(), firstStatementedNode);
        }
    }
};
function addReturnStatementImpl(sourceFile, node) {
    const statements = node.getStatements();
    // methodDecl.addStatements('return null;')  // this fails (  https://github.com/dsherret/ts-ast-viewer/issues/20)so we hack: 
    //TODO: should be fixed - update ts-simple-ast
    //TODO: use getDefaultValueForType to return an example value instead of null
    if (statements.length) {
        sourceFile.insertText(statements[statements.length - 1].getEnd(), '\nreturn null;');
    }
    else {
        sourceFile.insertText(node.getEnd() - 1, '\nreturn null;\n');
    }
}
//# sourceMappingURL=addReturnStatement.js.map