"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const ts_simple_ast_1 = require("ts-simple-ast");
/*

# description

reassigning a const variable is an error  this fix suggest changing it to let:

# example
```
const a = 1
a = 2
```
# Attack: 	"code": "2540",	"message": "Cannot assign to 'a' because it is a constant or a read-only property.",

  */
exports.const2let = {
    name: 'const2let',
    config: { changeTo: 'const' },
    predicate: (arg) => {
        if (!arg.diagnostics.find(d => d.code === 2540)) {
            return false;
        }
        if (arg.containingTargetLight.kind === ts.SyntaxKind.Identifier) {
            return true;
        }
        else {
            arg.log('codeFixCreateVariable predicate false because child.kind dont match ' + typescript_ast_util_1.getKindName(arg.containingTargetLight.kind));
            return false;
        }
    },
    description: (arg) => `Change "const ${arg.containingTarget.getText()}" to "let ${arg.containingTarget.getText()}"`,
    apply: (arg) => {
        const id = arg.simpleNode;
        if (!id || id.getKind() !== ts.SyntaxKind.Identifier) {
            arg.log(`codeFixCreateVariable apply cannot exec because of this !id||id.getKind()!== ts.SyntaxKind.Identifier  `);
            return;
        }
        else if (id.getParent() && id.getParent().getParent() && id.getParent().getParent().getKind() === ts.SyntaxKind.ExpressionStatement) {
            const declStatement = id.getSourceFile().getVariableStatement(v => v.getDeclarationKind() === ts_simple_ast_1.VariableDeclarationKind.Const && !!v.getDeclarations().find(vv => vv.getName() === id.getText()));
            declStatement.setDeclarationKind(ts_simple_ast_1.VariableDeclarationKind.Let);
        }
        else {
            arg.log(`codeFixCreateVariable apply cannot exec because this was false: id.getParent() && id.getParent()!.getParent() && id.getParent()!.getParent()!.getKind()===ts.SyntaxKind.ExpressionStatement `);
        }
    }
};
//# sourceMappingURL=const2let.js.map