"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
let newExpr;
exports.codeFixCreateConstructor = {
    name: 'Declare constructor',
    config: { variableType: 'const' },
    predicate: (arg) => {
        if (!arg.diagnostics.find(d => d.code === 2554)) {
            return false;
        }
        newExpr = ts.isNewExpression(arg.containingTarget) ? arg.containingTarget : typescript_ast_util_1.findAscendant(arg.containingTarget, ts.isNewExpression);
        if (newExpr) {
            return true;
        }
        else {
            arg.log(`codeFixCreateConstructor predicate false because no NewExpression ascendant was found containingTarget.kind==${typescript_ast_util_1.getKindName(arg.containingTarget.kind)}, containingTarget.parent.kind==${typescript_ast_util_1.getKindName(arg.containingTarget.parent.kind)}`);
            return false;
        }
    },
    description: (arg) => `Declare constructor "${newExpr.expression.getText()}"`,
    apply: (arg) => {
        const originalKind = arg.simpleNode.getKind();
        if (!ts_simple_ast_1.TypeGuards.isNewExpression(arg.simpleNode)) {
            arg.simpleNode = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.NewExpression);
        }
        if (!arg.simpleNode || !ts_simple_ast_1.TypeGuards.isNewExpression(arg.simpleNode)) {
            arg.log(`codeFixCreateConstructor apply fail because couldnt find a NewExpression from arg.simpleNode returned by sourcefile..getDescendantAtPos(positionOrRangeToNumber(positionOrRange) - returned arg.simpleNode kind was ${typescript_ast_util_1.getKindName(originalKind)}`);
            return;
        }
        const argTypes = arg.simpleNode.getArguments().map(arg => arg.getType().getApparentType().getText());
        const classDeclaration = arg.simpleNode.getExpression().getSymbol().getDeclarations()[0];
        if (ts_simple_ast_1.TypeGuards.isClassDeclaration(classDeclaration)) {
            classDeclaration.addConstructor({
                // docs: 'Autogenerated constructor', //TODO. add some jsdoc to generated constructor
                scope: ts_simple_ast_1.Scope.Public,
                parameters: argTypes.map(type => ({
                    name: `a${type.substring(0, 1).toUpperCase()}${type.substring(1, type.length)}`,
                    hasQuestionToken: false,
                    type,
                    isRestParameter: false,
                })),
                bodyText: `throw new Error('Not implemented');`
            });
        }
        else {
            arg.log(`codeFixCreateConstructor apply fail because arg.simpleNode is not ClassDeclaration is ${typescript_ast_util_1.getKindName(classDeclaration.getKind())}`);
        }
    }
};
//# sourceMappingURL=declareConstructor.js.map