"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const util_1 = require("../util");
/**

# Description:

When implementation miss ot implement a member or is doing it incorrectly, select member name and accept suggestion "implement interface"

# Diagnostic error attacked:

"code": "2416","message": "Property 'method1' in type 'SomeImplementation' is not assignable to the same property in base type 'SomeInterface'.\n  Type '(param: number) => number[]' is not assignable to type '(param: string) => number[]'.\n    Types of parameters 'param' and 'param' are incompatible.\n      Type 'string' is not assignable to type 'number'.",

# How to reproduce:
```
interface SomeInterface {
  prop1: { s: string, n: Date }[]
}
class Class2 implements SomeInterface {
  prop1: boolean[]        <----- here select prop1 and implement interface it will fix prop1 signature
}
```
# TODO

 * TODO: work for constructors and setter/getter members
 * TODO: review
 *
 * TODO: this is failing:
class Unit {
  energy: number
}
const unit: Unit = {
  energy: 123, color: 'red'
}

*/
exports.implementInterfaceMember = {
    name: 'implementInterfaceMember',
    config: { recursive: false, addMissingPropertiesToInterface: false },
    predicate: (arg) => {
        const targetLine = ts.getLineAndCharacterOfPosition(arg.sourceFile, arg.containingTarget.getStart()).line;
        const diagnostics = arg.diagnostics.filter(d => d.code === 2416).filter(diag => {
            const diagLineStart = ts.getLineAndCharacterOfPosition(arg.sourceFile, diag.start).line;
            const diagLineEnd = ts.getLineAndCharacterOfPosition(arg.sourceFile, diag.start + diag.length).line;
            return diagLineStart <= targetLine && diagLineEnd >= targetLine;
        });
        if (!diagnostics || !diagnostics.length) {
            arg.log('codeFixCreateVariable predicate false because no diagnostics found with code 2322 in same line as arg.containingTarget');
            return false;
        }
        if (arg.containingTargetLight.kind === ts.SyntaxKind.Identifier) {
            // in this case user selected a fragment of the id. quick issue fix: 
            if (arg.containedTarget && arg.containedTarget.kind === ts.SyntaxKind.SourceFile) {
                arg.containedTarget = undefined;
            }
            return true;
        }
        // else if (arg.containedTarget && arg.containedTarget.kind === ts.SyntaxKind.Identifier) {
        //   // user selected the exactly the id (double click)
        //   return true
        // }
        else if (arg.containedTarget && (typescript_ast_util_1.findAscendant(arg.containedTarget, n => n.kind === ts.SyntaxKind.PropertyDeclaration) || typescript_ast_util_1.findAscendant(arg.containedTarget, n => n.kind === ts.SyntaxKind.VariableDeclaration))) {
            return true;
        }
        else {
            arg.log('codeFixCreateVariable predicate false because child.kind dont match ' + typescript_ast_util_1.getKindName(arg.containingTarget.kind));
            return false;
        }
    },
    description: (arg) => `Implement Interface`,
    apply: (arg) => {
        const sourceFile = arg.simpleNode.getSourceFile();
        const id = arg.simpleNode;
        const member = id.getParent();
        const decl = member.getParent();
        if (!(ts_simple_ast_1.TypeGuards.isIdentifier(id) &&
            (ts_simple_ast_1.TypeGuards.isMethodDeclaration(member) || ts_simple_ast_1.TypeGuards.isPropertyDeclaration(member)) &&
            ts_simple_ast_1.TypeGuards.isClassDeclaration(decl))) {
            return arg.log(`predicate not complied: decl: ${decl.getKindName()} member: ${member.getKindName()} id: ${id.getKindName()}`);
        }
        const interfaceWithMemberName = util_1.findInterfacesWithPropertyNamed(decl, id.getText()).pop(); // TODO: we choose any member signature - we should choose the most similar one
        const memberSignature = interfaceWithMemberName.getMembers().filter(ts_simple_ast_1.TypeGuards.isPropertyNamedNode).pop(); // TODO: any arbitrary signature
        if (ts_simple_ast_1.TypeGuards.isMethodSignature(memberSignature) && ts_simple_ast_1.TypeGuards.isMethodDeclaration(member)) {
            util_1.fixSignature(member, memberSignature);
        }
        else if (ts_simple_ast_1.TypeGuards.isPropertySignature(memberSignature) && ts_simple_ast_1.TypeGuards.isPropertyDeclaration(member)) {
            member.setType(memberSignature.getType().getText());
        }
    }
};
//# sourceMappingURL=implementInterfaceMember.js.map