"use strict";
// Solves this two errors by adding properties to the object literal definition (or adding extra properties declared by literal to the intereface (f config.addMissingPropertiesToInterface===true))
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/*

# description

you explicit a type for a object literal that is not declared - this fix will sugest create the interface automatically inferring from that object literals

# example
```
const tree1: Living = { // 	"code": "2322",// 	"message": "Type '{}' is not assignable to type 'Living'.\n  Property 'name' is missing in type '{}'.",
}
interface Living {
  name: string
}
const tree2: Living = {
  name: 'n',
  dddd: 'hshs' //	"code": "2322",	"message": "Type '{ name: string; dddd: string; }' is not assignable to type 'Living'.\n  Object literal may only specify known properties, and 'dddd' does not exist in type 'Living'.",
}
```

# attacks
    "code": "2322",// 	"message": "Type '{}' is not assignable to type 'Living'.\n  Property 'name' is missing in type '{}'.",
*/
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const ts_simple_ast_1 = require("ts-simple-ast");
const util_1 = require("../util");
exports.implementInterfaceObjectLiteral = {
    name: 'implementInterfaceObjectLiteral',
    config: { recursive: false, addMissingPropertiesToInterface: false },
    predicate: (arg) => {
        const targetLine = ts.getLineAndCharacterOfPosition(arg.sourceFile, arg.containingTarget.getStart()).line;
        const diagnostics = arg.diagnostics.filter(d => d.code === 2322).filter(diag => {
            const diagLineStart = ts.getLineAndCharacterOfPosition(arg.sourceFile, diag.start).line;
            const diagLineEnd = ts.getLineAndCharacterOfPosition(arg.sourceFile, diag.start + diag.length).line;
            return diagLineStart <= targetLine && diagLineEnd >= targetLine;
        });
        if (!diagnostics || !diagnostics.length) {
            arg.log('implementInterfaceObjectLiteral predicate false because no diagnostics found with code 2322 in same line as arg.containingTarget');
            return false;
        }
        if (arg.containingTarget.kind === ts.SyntaxKind.Identifier) {
            // in this case user selected a fragment of the id. quick issue fix: 
            if (arg.containedTarget && arg.containedTarget.kind === ts.SyntaxKind.SourceFile) {
                arg.containedTarget = undefined;
            }
            return true;
        }
        else if (arg.containedTarget && arg.containedTarget.kind === ts.SyntaxKind.Identifier) {
            // user selected the exactly the id (double click)
            return true;
        }
        else if (arg.containedTarget && (typescript_ast_util_1.findAscendant(arg.containedTarget, n => n.kind === ts.SyntaxKind.PropertyDeclaration) || typescript_ast_util_1.findAscendant(arg.containedTarget, n => n.kind === ts.SyntaxKind.VariableDeclaration))) {
            return true;
        }
        else {
            arg.log('implementInterfaceObjectLiteral predicate false because child.kind dont match ' + typescript_ast_util_1.getKindName(arg.containingTarget.kind));
            return false;
        }
    },
    description: (arg) => `Implement Interface`,
    apply: (arg) => {
        const node = arg.simpleNode;
        const varDecl = ts_simple_ast_1.TypeGuards.isVariableDeclaration(node) ? node : node.getFirstAncestorByKind(ts.SyntaxKind.VariableDeclaration);
        const init = varDecl.getInitializerIfKind(ts.SyntaxKind.ObjectLiteralExpression);
        varDecl.getType().getSymbol().getDeclarations().forEach(decl => {
            if (!ts_simple_ast_1.TypeGuards.isInterfaceDeclaration(decl)) {
                return; // TODO: log
            }
            const toRemove = [];
            // init.getProperties().forEach(prop => {
            //   //  -TODO: this doesn't work here : TypeGuards.isNameableNode(prop) so we ugly cast 
            //   if ((prop as any).getName && !decl.getMembers().find(m => (m as any).getName() === (prop as any).getName())) {            
            //     // if(this.config.addMissingPropertiesToInterface){ // TODO: doesn't work 
            //     // decl.addProperty({name: prop.getName(), type: prop.getType().getText()})
            //     // } else {
            //     toRemove.push(prop)
            //   }
            // })
            decl.getProperties().forEach(prop => {
                const existingProp = init.getProperty(prop.getName());
                if (existingProp && existingProp.remove) {
                    existingProp.remove();
                }
                else if (existingProp && !existingProp.remove) { // TODO : fixed with https://github.com/dsherret/ts-simple-ast/pull/343#issuecomment-394923115 - remove will be always present
                    arg.log(`implementInterfaceObjectLiteral apply WARNING existingProp &&  !(existingProp as any).remove: kind: ${existingProp.getKindName()} text: ${existingProp.getText()} init.getText() === ${init.getText()}`);
                }
                else {
                    init.addPropertyAssignment({ name: prop.getName(), initializer: util_1.getDefaultValueForType(prop.getType()) });
                }
            });
            decl.getMethods().forEach(method => {
                const existingProp = init.getProperty(method.getName());
                // if (!TypeGuards.isMethodDeclaration(existingProp)) {
                //   return
                // }
                // if (existingProp) {
                //   (existingProp as any).remove()
                // }
                // let parameters = method.getParameters().map(buildParameterStructure)
                if (existingProp && ts_simple_ast_1.TypeGuards.isMethodDeclaration(existingProp)) {
                    util_1.fixSignature(existingProp, method);
                    arg.log('fixSignature1');
                }
                else if (existingProp) {
                    // try{
                    existingProp.remove();
                    //   }catch(ex){
                    //     arg.log('fixSignature222')}
                    // }
                    init.addMethod({
                        name: method.getName(),
                        parameters: method.getParameters().map(util_1.buildParameterStructure),
                        returnType: method.getReturnType().getText(),
                        bodyText: 'throw new Error(\'Not Implemented\')'
                    });
                }
            });
            // only remove if user explicitly configure
            // if(this.config.removeStrangeMembersFromLiteral){ // TODO: doesn't work 
            // toRemove.forEach(prop => {
            //   prop.getSourceFile().removeText(prop.getFullStart(),
            //     prop.getNextSibling() && prop.getNextSibling().getKind() === ts.SyntaxKind.CommaToken ? prop.getNextSibling().getEnd() : prop.getEnd())
            // })
            // }
        });
    }
};
//# sourceMappingURL=implementInterfaceObjectLiteral.js.map